# iOS拍摄SDK开发文档

## 介绍

视频云以SDK形式提供视频拍摄功能 ，帮助开发者快速实现视频拍摄能力。SDK包括开发文档、SDK。开发者可参考文档，将相关framework加入工程中，完成相关配置，调用相关的API即可接入拍摄功能。

## 功能说明

拍摄包括采集（音频、视频）、渲染（Metal/OpenGL）、分段录制、合成等主要功能。

| 功能列表| 
| -------- |
| 视频采集 | 
| 音频采集 | 
| 自定义分辨率、码率、帧率 | 
| 照片拍摄 | 
| 分段录制 | 
| 倍速| 
| 回删 | 
| 分段合成 | 
| 滤镜 | 
| 背景音乐 .etc | 

## 系统范围

| 系统特性 | 支持范围     |
| -------- | ------------ |
| 系统版本 | iOS8+        |
| 系统架构 | armv7、armv7s、arm64 |



## SDK集成

### 配置说明

1. 拍摄功能提供两个framework：

	QHVCShortVideoKit.framework该库为动态库（Build Phases->Embed Frameworks-> +）

	QHVCCommonKit.framework该库为动态库（Build Phases->Embed Frameworks-> +）




2. 实际开发中#import `<QHVCShortVideoKit/QHVCShortVideoKit.h>`头文件调用相关接口。

#### SDK初始化

需要调用`[QHVCCommonCoreEntry coreOnAppStart:appId
                                 appVer:version
                               deviceId:deviceId
                                  model:model
                         optionalParams:nil];`接口，说明如下：

appId(也叫businessId)是为业务分配的业务ID（appId获取地址：登录[视频云官网](https://zyun.360.cn/)-控制台-SDK-应用管理-新建应用）【见下图】；version为业务端APP的版本号；deviceId为设备唯一标识；uid为业务方用户标识，只要保证唯一即可;options可选，通常传nil，如有特殊需求详见参数说明。


![创建AppId截图](http://p5.qhimg.com/t011da35ea7b6063845.png)

[点此查看应用ID](https://zyun.360.cn/console/sdk/application)

> 对appId和channelID这两个参数的引入，在设计之初目的是为了每个业务有独立的ID，通过appId来获取云端的一些参数控制，特别是视频缓冲策略、P2P行为、硬解码黑白名单以及解码库类型；channelID更多的为数据统计打点的唯一标注，可以通过云点播和云直播的服务标识来获取，以此获取播放数据反馈。UserID的设计是为了业务方和视频云唯一的沟通桥梁，只要保持唯一即可，一般为用户ID的加密字符串，通过UserID可以排查到一个用户的级别的失败日志；

#### APPID合法性验证

需要调用 `[QHVCCommonCoreEntry validityCheck:token randNum:randNum authTime:authTime callback:^(QHVCAuthStatus ret, NSString *detail) {
    }];，接口说明如下：

合法性验证需要用到appId, Access Key(简称AK)和Secret Key(简称SK)。AK、SK获取地址： 登录[视频云官网](https://zyun.360.cn/)-用户中心-秘钥管理【见下图】。
只有appId、Access Key以及Secret Key相匹配，才能通过鉴权，否则将无法正常使用SDK。

![秘钥获取](http://p2.qhimg.com/t0146c3d44e71ae7a62.png)

[点此查看秘钥](https://zyun.360.cn/console/usercenter/accesskey)

==注意：此接口必须在启动APP后120秒内调用，否则将无法正常使用SDK。==

appId使用和SDK初始化时同样的内容，callback可以用来监听合法性验证结果。 计算authorization时需要使用AK、SK以及当前系统的时间戳和随机数。
**为了保证AK、SK的安全，请业务接入时，务必把AK和SK存储在服务端，并且把authorization的计算也放在服务端来做。**

计算authorization的示例代码如下：


```
    NSString *randNum = [NSString stringWithFormat:@"%u", arc4random() + 1];
    NSString *authTime = [NSString stringWithFormat:@"%lld", [QHVCToolUtils getCurrentDateByMilliscond]/1000];
    QHVCGlobalConfig* globalConfig = [QHVCGlobalConfig sharedInstance];//参数配置
    NSString *AK = globalConfig.appKey;
    NSString *SK = globalConfig.appSecret;
    NSString *paramsStr = [NSString stringWithFormat:@"appid=%@", [QHVCGlobalConfig sharedInstance].appId];
    NSString *paramsSign = [QHVCToolUtils getMD5String:[NSString stringWithFormat:@"%@%@", [QHVCToolUtils getMD5String:paramsStr], randNum]];
    NSString *authString = [NSString stringWithFormat:@"%@\n%@\n%@\n%@", AK, authTime, randNum, paramsSign];
    NSString *sign = [QHVCToolUtils hmacSha1:SK data:authString];
    NSString *token = [NSString stringWithFormat:@"%@:%@", AK, sign];
```

鉴权状态码：

```
typedef NS_ENUM(NSInteger, QHVCAuthStatus)
{
    QHVCAuthStatus_OK = 1,                //鉴权成功
    QHVCAuthStatus_FAILED = -11001,       //鉴权失败
    QHVCAuthStatus_NOT_INVOKED = -11002   //未调用鉴权接口
};
```
## 接口说明

### 初始化

创建拍摄对象（注意：初始化sdk前，需要获取相机、麦克风访问权限，否则会有异常）


```

[QHVCRecord openLogWithLevel:QHVCRecordLogLevelTrace];//log for test
    
    _videoSession = [[QHVCRecord alloc] init];
    [_videoSession setStatisticsInfo:@{@"channelId":@"demo_1",
                                       @"userId":@"110",
                                       }];//统计相关
    [_videoSession setVideoConfig:[QHVCRecordVideoConfig defaultVideoConfig]];//设置视频编码输出参数
    [_videoSession setAudioConfig:[QHVCRecordAudioConfig defaultAudioConfig]];//设置音频编码输出参数

    [_videoSession setRecordDelegate:self];
//    [_videoSession switchCamera:_isFrontCamera];
    [_videoSession startCameraPreview:preview];//开始本地预览
```

### 分段录制


```

if (_isRecording) {
    _recordBtn.enabled = NO;
    [_videoSession stopRecord];
}
else
{
    // Disable the idle timer while recording
    [UIApplication sharedApplication].idleTimerDisabled = YES;
    
    // Make sure we have time to finish saving the movie if the app is backgrounded during recording
    if ( [[UIDevice currentDevice] isMultitaskingSupported] ) {
        _backgroundRecordingID = [[UIApplication sharedApplication] beginBackgroundTaskWithExpirationHandler:^{}];
    }
    
    if (!_isFirstRecord) {
        BOOL status = [[NSFileManager defaultManager] removeItemAtPath:_cacheFolder error:nil];//每次录制前或者结束后清空cacheFolder
        if (!status) {
            NSLog(@"remove fail !!!");
        }
        [_videoSession setRecordPath:_outputFile videoSegmentsFolder:_cacheFolder];
        _isFirstRecord = YES;
    }
    [_videoSession setRecordSpeed:self.recordSpeed];//设置倍速
    
    _recordBtn.enabled = NO;// avoid re-enabled
    [_videoSession startRecord];//开始录制
}
        
```
###合成
```
合成进度、状态等信息获取，参见QHVCRecordDelegate.h

[_videoSession joinAllSegments];

```

```

## 错误码说明

|状态码|含义|
|:--:|:--|
|-999|时间戳错误|
|-899|参数错误|
|-797|创建线程失败|
|-796|打开文件失败|
|-794|打开编码器失败|