//
//  QHVCEffectDefinitions.h
//  QHVCShortVideoKit
//
//  Created by liyue-g on 2019/9/3.
//  Copyright © 2019 qihoo.QHVC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreGraphics/CoreGraphics.h>

#pragma mark - QHVCEffectRect

struct
QHVCEffectRect {
    CGPoint center;  //中心点 (相对坐标0-1)
    CGSize  size;    //宽高（相对尺寸0-1）
};
typedef struct QHVCEffectRect QHVCEffectRect;

#ifdef __cplusplus
extern "C" {
#endif
    QHVCEffectRect QHVCEffectRectMake(CGFloat x, CGFloat y, CGFloat width, CGFloat height);
    
#ifdef __cplusplus
}
#endif

#define QHVCEffectRectZero QHVCEffectRectMake(0,0,0,0)


typedef NS_ENUM(NSUInteger, QHVCEffectRenderParamType)
{
    QHVCEffectRenderParamTypeAlpha,     //透明度（0-1），默认1.0
    QHVCEffectRenderParamTypeScale,     //缩放，默认1.0
    QHVCEffectRenderParamTypeX,         //相对画布的x坐标，默认0
    QHVCEffectRenderParamTypeY,         //相对画布的y坐标，默认0
    QHVCEffectRenderParamTypeRadian,    //旋转弧度值，默认0
};

typedef NS_ENUM(NSUInteger, QHVCEffectCurveType)
{
    QHVCEffectCurveTypeLinear,
    QHVCEffectCurveTypeCatmullRom,
};

typedef NS_ENUM(NSUInteger, QHVCEffectBlendType)
{
    QHVCEffectBlendTypeAlpha,            //alpha混合
    QHVCEffectBlendTypeOverlay,          //叠加
    QHVCEffectBlendTypeMultiply,         //多倍
    QHVCEffectBlendTypeScreen,           //屏幕
    QHVCEffectBlendTypeSoftLight,        //柔光
    QHVCEffectBlendTypeHardLight,        //强光
    QHVCEffectBlendTypeDarken,           //变暗
    QHVCEffectBlendTypeColorBurn,        //颜色加深
    QHVCEffectBlendTypeLighten,          //变亮
    QHVCEffectBlendTypeLinearDodge,      //更亮
    QHVCEffectBlendTypeLinearBurn,       //更暗
};

typedef NS_ENUM(NSUInteger, QHVCEffectQualityType)
{
    QHVCEffectQualityTypeBrightness,       //亮度 (参数范围-100~100)
    QHVCEffectQualityTypeContrast,         //对比度 (参数范围-100~100)
    QHVCEffectQualityTypeExposure,         //曝光度 (参数范围-100~100)
    QHVCEffectQualityTypeGammaOffset,      //gamma补偿 (参数范围-100~100)
    QHVCEffectQualityTypeTemperature,      //色温 (参数范围-100~100)
    QHVCEffectQualityTypeTint,             //色调 (参数范围-100~100)
    QHVCEffectQualityTypeSaturation,       //饱和度 (参数范围-100~100)
    QHVCEffectQualityTypeHue,              //色相 (参数范围-180~180)
    QHVCEffectQualityTypeVibrance,         //鲜艳度（自然饱和度）(参数范围-100~100)
    QHVCEffectQualityTypeVignette,         //暗角 (参数范围0~100)
    QHVCEffectQualityTypePrism,            //色散 (参数范围0~100)
    QHVCEffectQualityTypeFade,             //褪色 (参数范围0~100)
    QHVCEffectQualityTypeHighlight,        //高光减弱 (参数范围0~100)
    QHVCEffectQualityTypeShadow,           //阴影补偿 (参数范围0~100)
    QHVCEffectQualityTypeSkin,             //肤色（参数范围-100~100）
    QHVCEffectQualityTypeSharpen,          //锐度（参数范围0~100）
    QHVCEffectQualityTypeFilmGrain,        //颗粒噪声（参数范围0~100）
    QHVCEffectQualityTypeBlur,             //模糊（参数范围0~100）
    QHVCEffectQualityTypeHighlightOrange,  //高光色调橙色（参数范围0~100）
    QHVCEffectQualityTypeHighlightCream,   //高光色调奶油色（参数范围0~100）
    QHVCEffectQualityTypeHighlightYellow,  //高光色调黄色（参数范围0~100）
    QHVCEffectQualityTypeHighlightGreen,   //高光色调绿色（参数范围0~100）
    QHVCEffectQualityTypeHighlightBlue,    //高光色调蓝色（参数范围0~100）
    QHVCEffectQualityTypeHighlightMagenta, //高光色调洋红色（参数范围0~100）
    QHVCEffectQualityTypeShadowRed,        //阴影色调红色（参数范围0~100）
    QHVCEffectQualityTypeShadowOrange,     //阴影色调橙色（参数范围0~100）
    QHVCEffectQualityTypeShadowYellow,     //阴影色调黄色（参数范围0~100）
    QHVCEffectQualityTypeShadowGreen,      //阴影色调绿色（参数范围0~100）
    QHVCEffectQualityTypeShadowBlue,       //阴影色调蓝色（参数范围0~100）
    QHVCEffectQualityTypeShadowPurple,     //阴影色调紫色（参数范围0~100）
};
